﻿<?xml version="1.0" encoding="UTF-8" ?>

<!--

Help Insight Transformation Template V0.9.2

>> Copyright
Author:     DevJET Software
Modified:   Feb 7th, 2011

Copyright (c) 2011 DevJET Software
http://www.devjet.net

>> Introduction

This XSLT template is used to transform xml documentation in source code to
html page which will be displayed in the popup Help Insight window.

Goals:
* Support most tags in Delphi Documentation Guidelines
* Pass through common html tags
* Better looking and feel
* Configurable

>> Change Log

V0.9.2  Feb 7th, 2011
# Fixed the encoding issue

-->

<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:di="http://www.devjet.net/di">

<!-- Output: XHTML 1.0 -->
<xsl:output method="html" encoding="UTF-8"
    doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"
    indent="yes" />

<!-- Global configuration -->
<xsl:variable name="configurations" select="document('HelpInsight\Configurations.xml')/configuration" />
<xsl:variable name="noteConfigurations" select="$configurations/notes" />

<!-- Highlight the unresolved elements -->
<xsl:template match="*">
    <span style="background-color:yellow">
        <xsl:attribute name="title">&lt;<xsl:value-of select="."/>&gt;</xsl:attribute>
        <xsl:apply-templates/>
    </span>
</xsl:template>


<!-- Pass through most common html tags -->
<xsl:template match="p|ol|ul|li|dl|dt|dd|table|tr|th|td|a|img|b|i|strong|em|del|sub|sup|br|hr|h1|h2|h3|h4|h5|h6|pre|div|span|blockquote|abbr|acronym|u|font|map|area">
	<xsl:copy>
		<xsl:copy-of select="@*" />
		<xsl:apply-templates />
	</xsl:copy>
</xsl:template>


<!-- Section Tags -->
<!--
    <summary>
    <typeparam>
    <param>
    <returns>
    <value>
    <exception>
    <permission>
    <remarks>
    <comments>
    // <example>
    // <seealso>
    // <include>
    // <preliminary>
    // <threadsafety>
-->

<xsl:template match="summary|remarks|value|returns|comments">
    <xsl:choose>
        <xsl:when test="para|p">
            <xsl:apply-templates select="./node()" />
        </xsl:when>
        <xsl:otherwise>
            <p>
                <xsl:apply-templates select="./node()" />
            </p>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template match="typeparam|param">
	<dt>
		<span>
            <xsl:attribute name="class">
                <xsl:value-of select="local-name()" />
            </xsl:attribute>
            <xsl:value-of select="@name"/>
        </span>
	</dt>
	<dd>
		<xsl:apply-templates />
	</dd>
</xsl:template>

<xsl:template match="exception|permission">
	<dt>
        <a>
            <xsl:attribute name="href">
                <xsl:text>helpinsight:/typelink:</xsl:text>
                <xsl:value-of select="@cref" />
            </xsl:attribute>

            <xsl:choose>
                <xsl:when test="string(@DisplayName)!=string(@cref)">
                    <xsl:value-of select="@DisplayName" />
                </xsl:when>
                <xsl:when test="contains(@cref, '|')">
                    <xsl:value-of select="translate(substring-after(@cref, '|'), '{}', '&lt;&gt;')" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="translate(@cref, '{}', '&lt;&gt;')" />
                </xsl:otherwise>
            </xsl:choose>
        </a>
	</dt>
	<dd>
		<xsl:apply-templates />
	</dd>
</xsl:template>

<!-- Block Tags -->
<!--
    <para>
    <code>
    <list>
    <note>
-->

<xsl:template match="para">
	<p>
    	<xsl:apply-templates />
	</p>
</xsl:template>

<xsl:template match="code">
    <div class="code">
        <pre class="code">
            <xsl:apply-templates />
        </pre>
    </div>
</xsl:template>

<xsl:template match="list[@type='bullet']">
    <ul>
        <xsl:for-each select="item">
            <li>
                <xsl:apply-templates />
            </li>
        </xsl:for-each>
    </ul>
</xsl:template>

<xsl:template match="list[@type='number']">
    <ol>
        <xsl:for-each select="item">
            <li>
                <xsl:apply-templates />
            </li>
        </xsl:for-each>
    </ol>
</xsl:template>

<xsl:template match="list[@type='table']">
    <table class="definitionTable">
        <xsl:if test="listheader">
            <thead>
                <tr>
                    <td class="termColumn"><xsl:value-of select="listheader/term" /></td>
                    <td class="descriptionColumn"><xsl:value-of select="listheader/description" /></td>
                </tr>
            </thead>
        </xsl:if>
        <tbody>
            <xsl:for-each select="item">
                <tr>
                    <td class="term"><p><xsl:apply-templates select="term/node()" /></p></td>
                    <td class="description"><p><xsl:apply-templates select="description/node()" /></p></td>
                </tr>
            </xsl:for-each>
        </tbody>
    </table>
</xsl:template>

<xsl:template match="note">
    <xsl:variable name="value" select="@type" />
    <xsl:variable name="option" select="$noteConfigurations/node()[name()=$value]" />
    <xsl:call-template name="insertNote">
        <xsl:with-param name="type" select="@type" />
        <xsl:with-param name="title" select="$option/title" />
        <xsl:with-param name="iconUri">
            <xsl:text>HelpInsight\Icons\</xsl:text>
            <xsl:value-of select="$option/iconUri" />
        </xsl:with-param>
    </xsl:call-template>
</xsl:template>

<xsl:template name="insertNote">
    <xsl:param name="type" />
    <xsl:param name="iconUri" />
    <xsl:param name="title" select="'Note:'" />
    <div class="note">
        <table>
            <thead>
            <tr>
                <th>
                    <img src="{$iconUri}" class="note" title="Note" />
                    <span class="note"><xsl:value-of select="$title" /></span>
                </th>
            </tr>
            </thead>
            <tbody>
                <tr>
                    <td>
                        <div>
                            <xsl:choose>
                                <xsl:when test="para|p">
                                    <xsl:apply-templates />
                                </xsl:when>
                                <xsl:otherwise>
                                    <p>
                                        <xsl:apply-templates />
                                    </p>
                                </xsl:otherwise>
                            </xsl:choose>
                        </div>
                    </td>
                </tr>
            </tbody>
        </table>
    </div>
</xsl:template>


<!-- Inline Tags -->

<!--
    <c>
    <paramref>
    <typeparamref>
    <see>
-->

<xsl:template match="c">
	<tt class="inlineCode">
        <xsl:apply-templates/>
    </tt>
</xsl:template>

<xsl:template match="paramref">
	<span class="paramref">
    	<xsl:value-of select="@name"/>
    </span>
</xsl:template>

<xsl:template match="typeparamref">
	<span class="typeparamref">
        <xsl:value-of select="@name"/>
    </span>
</xsl:template>

<xsl:template match="see[@cref]">
    <a class="seecref" href="helpinsight:/typelink:{@cref}">
        <xsl:attribute name="title">
            <xsl:value-of select="translate(@cref, '|{}', '.&lt;&gt;')" />
        </xsl:attribute>
        <xsl:choose>
            <xsl:when test="normalize-space(.)">
                <xsl:value-of select="." />
            </xsl:when>
            <xsl:when test="string(@DisplayName)!=string(@cref)">
                <xsl:value-of select="@DisplayName" />
            </xsl:when>
            <xsl:when test="contains(@cref, '|')">
                <xsl:value-of select="translate(substring-after(@cref, '|'), '{}', '&lt;&gt;')" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="translate(@cref, '{}', '&lt;&gt;')" />
            </xsl:otherwise>
        </xsl:choose>
    </a>
</xsl:template>

<xsl:template match="see[@href]">
    <a target="_blank" href="{@href}" title="{@href}">
        <xsl:choose>
            <xsl:when test="normalize-space(.)">
                <xsl:value-of select="." />
            </xsl:when>
            <xsl:when test="@DisplayName">
                <xsl:value-of select="@DisplayName" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="@href" />
            </xsl:otherwise>
        </xsl:choose>
    </a>
</xsl:template>

<xsl:template match="see[@langword]">
    <a class="seelangword">
        <xsl:attribute name="href">
            langword:<xsl:value-of select="@langword" />
        </xsl:attribute>
        <xsl:choose>
            <xsl:when test="normalize-space(.)">
                <xsl:value-of select="." />
            </xsl:when>
            <xsl:when test="@DisplayName">
                <xsl:value-of select="@DisplayName" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="@langword" />
            </xsl:otherwise>
        </xsl:choose>
    </a>
</xsl:template>

<!-- Shared Templates -->

<xsl:template name="insertElementLink">
    <xsl:param name="element" />

    <a>
        <xsl:attribute name="href">
            helpinsight:/typelink:<xsl:value-of select="element/@cref"/>
        </xsl:attribute>

        <xsl:if test="element/@DisplayName">
            <xsl:value-of select="element/@DisplayName" />
        </xsl:if>

        <xsl:if test="string(element/@DisplayName) = ''">
            <xsl:value-of select="element/@cref" />
        </xsl:if>
    </a>
</xsl:template>

<!-- Main Entry -->

<xsl:template match="/">
    <html>
        <head>
            <link type='text/css' rel='Stylesheet' href='HelpInsight/HelpInsight.css' />
            <title>
                <xsl:value-of select="member/@DisplayName"/>
            </title>
        </head>
        <body>
            <xsl:call-template name="insertHeader" />
            <xsl:call-template name="insertContent" />
        </body>
    </html>
</xsl:template>


<xsl:template name="insertHeader">

    <table
        background="HelpInsightGradient.gif"
        border="0"
        width="100%"
        cellpadding="0"
        cellspacing="0">
        <tr>
            <td nowrap="true">
                <div id="mainCaption">
                    <xsl:choose>
                        <xsl:when test="member/source/@declaredIn">
                            <a class="goToDefinition" href="#">
                                <xsl:attribute name="href">
                                    <xsl:text>helpinsight:/filelink:</xsl:text>
                                    <xsl:value-of select="member/source/@declaredIn"/>
                                    <xsl:text>?</xsl:text>
                                    <xsl:value-of select="member/source/@declaredOn"/>
                                </xsl:attribute>
                                <xsl:value-of select="member/@DisplayName"/>
                                <xsl:attribute name="title">
                                    <xsl:value-of select="member/source/@declaredInShort"/> (<xsl:value-of select="member/source/@declaredOn"/>)
                                </xsl:attribute>
                            </a>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="member/@DisplayName"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </div>
            </td>
        </tr>
    </table>

    <!--
        <div id="title">
            <xsl:value-of select="member/@DisplayName"/>
        </div>
    -->
</xsl:template>

<xsl:template name="insertContent">
    <div id="content">
        <xsl:if test="count(member/summary) > 0">
            <div id="summary" class="section">
                <!--
                <xsl:if test="count(member/summary) > 1">
                    <h4>Summary</h4>
                </xsl:if>
                -->
                <div class="sectionContent">
                    <xsl:apply-templates select="member/summary[1]"/>
                </div>
            </div>
        </xsl:if>

        <xsl:if test="count(member/param) > 0">
            <div id="parameters" class="section">
                <h4>Parameters</h4>
                <div class="sectionContent">
                    <dl>
                        <xsl:for-each select="member/param">
                            <xsl:apply-templates select="."/>
                        </xsl:for-each>
                    </dl>
                </div>
            </div>
        </xsl:if>

        <xsl:if test="count(member/returns) > 0">
            <h4>Returns</h4>
            <div id="returns" class="sectionContent">
                <xsl:apply-templates select="member/returns"/>
            </div>
        </xsl:if>

        <xsl:if test="count(member/value) > 0">
            <h4>Value</h4>
            <div id="value" class="sectionContent">
                <xsl:apply-templates select="member/value"/>
            </div>
        </xsl:if>

        <xsl:if test="count(member/exception) > 0">
            <h4>Exceptions</h4>
            <div id="exceptions" class="sectionContent">
                <dl>
                    <xsl:for-each select="member/exception">
                        <xsl:apply-templates select="."/>
                    </xsl:for-each>
                </dl>
            </div>
        </xsl:if>

        <xsl:if test="count(member/permission) > 0">
            <h4>Permission</h4>
            <div id="permission" class="sectionContent">
                <dl>
                    <xsl:for-each select="member/permission">
                        <xsl:apply-templates select="."/>
                    </xsl:for-each>
                </dl>
            </div>
        </xsl:if>

        <xsl:if test="member/remarks">
            <h4>Remarks</h4>
            <div class="sectionContent">
                <xsl:apply-templates select="member/remarks"/>
            </div>
        </xsl:if>

        <xsl:if test="member/comments">
            <h4>Comments</h4>
            <div class="sectionContent">
                <xsl:for-each select="member/comments">
                    <xsl:apply-templates select="."/>
                </xsl:for-each>
            </div>
        </xsl:if>
    </div>
</xsl:template>

</xsl:stylesheet>
